import { defineComponent, PropType, reactive, ref, toRef, watch } from 'vue';
import styles from './index.module.less';
import dayjs from 'dayjs';
import { Button, DatePicker, Popup, showToast } from 'vant';
import { formatterDatePicker } from '@/helpers/utils';

export type TIME_TYPE = 'MONTH' | 'THREE_MONTH' | 'HALF_YEAR' | 'YEAR' | '';

/** 获取时间范围 */
export const getTimeRange = (type: TIME_TYPE) => {
  if (type === 'MONTH') {
    return {
      startTime: dayjs().format('YYYY-MM') + '-01',
      endTime: dayjs().format('YYYY-MM-DD')
    };
  } else if (type === 'THREE_MONTH') {
    return {
      startTime: dayjs().subtract(3, 'month').format('YYYY-MM-DD'),
      endTime: dayjs().format('YYYY-MM-DD')
    };
  } else if (type === 'HALF_YEAR') {
    return {
      startTime: dayjs().subtract(6, 'month').format('YYYY-MM-DD'),
      endTime: dayjs().format('YYYY-MM-DD')
    };
  } else if (type === 'YEAR') {
    return {
      startTime: dayjs().subtract(1, 'year').format('YYYY-MM-DD'),
      endTime: dayjs().format('YYYY-MM-DD')
    };
  }
};

/** 获取时间，拆分成数组 */
export const getTimeArray = (time?: string) => {
  const currentTime = time ? dayjs(time) : dayjs();
  return [
    currentTime.format('YYYY'),
    currentTime.format('MM'),
    currentTime.format('DD')
  ];
};

export default defineComponent({
  name: 'the-time-range',
  props: {
    /** 是否显示 */
    show: {
      type: Boolean,
      default: false
    },
    /** 默认时间范围 */
    typeUnit: {
      type: String as PropType<TIME_TYPE>,
      default: ''
    }
  },
  emits: ['close', 'confirm', 'update:show'],
  setup(props, { emit }) {
    const showPopoverTime = ref(toRef(props.show));
    const searchObj = reactive({
      type: ''
    });
    const timeRange = getTimeRange('');

    const forms = reactive({
      startTimeStatus: false,
      startTimeClosedStatus: false,
      endTimeMinDate: timeRange?.startTime
        ? new Date(timeRange?.startTime)
        : new Date(),
      endTimeMaxDate: dayjs(
        timeRange?.startTime ? new Date(timeRange?.startTime) : new Date()
      )
        .add(1, 'year')
        .toDate(),
      endTimeStatus: false,
      endTimeClosedStatus: false,
      startTime: getTimeArray(timeRange?.startTime),
      startTimeStr: timeRange?.startTime || '',
      endTime: getTimeArray(timeRange?.endTime),
      endTimeStr: timeRange?.endTime || ''
    });

    const onChangeTime = (type: TIME_TYPE) => {
      if (searchObj.type === type) return;
      searchObj.type = type;

      resetTime(type);
    };
    // 格式化
    const resetTime = (type: TIME_TYPE) => {
      const timeRang = getTimeRange(type);

      forms.startTime = getTimeArray(timeRang?.startTime);
      forms.startTimeStr = timeRang?.startTime || '';
      forms.endTimeMinDate = dayjs(timeRang?.startTime || '').toDate();
      forms.endTimeMaxDate = dayjs(timeRang?.startTime || '')
        .add(1, 'year')
        .toDate();
      forms.endTime = getTimeArray(timeRang?.endTime);
      forms.endTimeStr = timeRang?.endTime || '';
    };

    // 确认
    const onConfirm = () => {
      if (forms.startTimeStr && !forms.endTimeStr || !forms.startTimeStr && forms.endTimeStr) {
        showToast('请选择时间范围');
        return;
      }

      showPopoverTime.value = false
      // 确认
      emit('confirm', {
        startTime: forms.startTimeStr,
        endTime: forms.endTimeStr
      });
    };

    watch(() => props.show, () => {
      showPopoverTime.value = props.show
    })

    return () => (
      <div>
        <Popup
          v-model:show={showPopoverTime.value}
          closeable
          round
          position="bottom"
          class={styles.searchPopup}
          onClosed={() => {
            emit("update:show", false)
          }}>
          <div class={styles.popupContainer}>
            <div class={styles.popupTitle}>筛选</div>

            <div class={styles.popupSearchList}>
              <div class={styles.popupSection}>
                <div class={styles.title}>
                  <span>时间</span>
                </div>

                <div class={styles.timeCount}>
                  <p
                    onClick={() => onChangeTime('MONTH')}
                    class={searchObj.type === 'MONTH' ? styles.active : ''}>
                    本月
                  </p>
                  <p
                    onClick={() => onChangeTime('THREE_MONTH')}
                    class={
                      searchObj.type === 'THREE_MONTH' ? styles.active : ''
                    }>
                    近三个月
                  </p>
                  <p
                    onClick={() => onChangeTime('HALF_YEAR')}
                    class={searchObj.type === 'HALF_YEAR' ? styles.active : ''}>
                    近半年
                  </p>
                  <p
                    onClick={() => onChangeTime('YEAR')}
                    class={searchObj.type === 'YEAR' ? styles.active : ''}>
                    近一年
                  </p>
                </div>

                <div class={styles.timeRang}>
                  <p
                    class={[
                      styles.timeInput,
                      forms.startTimeStr && styles.hasValue
                    ]}
                    onClick={() => {
                      forms.startTimeStatus = true;
                      forms.startTimeClosedStatus = true;

                      console.log(forms.startTime, 'startTime');
                    }}>
                    {forms.startTimeStr || '起始时间'}
                  </p>
                  <p class={styles.timeUnit}></p>
                  <p
                    class={[
                      styles.timeInput,
                      forms.endTimeStr && styles.hasValue
                    ]}
                    onClick={() => {
                      forms.endTimeStatus = true;
                      forms.endTimeClosedStatus = true;
                    }}>
                    {forms.endTimeStr || '终止时间'}
                  </p>
                </div>
              </div>
            </div>

            <div class={styles.popupBottom}>
              <Button
                round
                block
                type="default"
                onClick={() => {
                  searchObj.type = '';
                  resetTime('');
                }}>
                重置
              </Button>
              <Button round block type="primary" onClick={onConfirm}>
                确认
              </Button>
            </div>
          </div>
        </Popup>

        {/* 开始日期 */}
        <Popup
          v-model:show={forms.startTimeStatus}
          position="bottom"
          round
          class={'popupBottomSearch'}
          onClosed={() => {
            forms.startTimeClosedStatus = false;
          }}>
          {forms.startTimeClosedStatus && (
            <DatePicker
              v-model={forms.startTime}
              formatter={formatterDatePicker}
              onCancel={() => (forms.startTimeStatus = false)}
              onConfirm={(val: any) => {
                console.log(val, 'val');
                forms.startTime = val.selectedValues;
                const valTime = val.selectedValues.join('-');
                forms.startTimeStr = dayjs(valTime).format('YYYY-MM-DD');
                forms.startTimeStatus = false;

                forms.endTimeMinDate = dayjs(valTime || new Date()).toDate();
                forms.endTimeMaxDate = dayjs(valTime || new Date())
                  .add(1, 'year')
                  .toDate();
                forms.endTime = val.selectedValues;
                forms.endTimeStr = '';
                searchObj.type = '' as any;
              }}
            />
          )}
        </Popup>
        {/* 结束日期 */}
        <Popup
          v-model:show={forms.endTimeStatus}
          position="bottom"
          round
          class={'popupBottomSearch'}
          onClosed={() => {
            forms.endTimeClosedStatus = false;
          }}>
          {forms.endTimeClosedStatus && (
            <DatePicker
              v-model={forms.endTime}
              minDate={forms.endTimeMinDate}
              maxDate={forms.endTimeMaxDate}
              formatter={formatterDatePicker}
              onCancel={() => (forms.endTimeStatus = false)}
              onConfirm={(val: any) => {
                forms.endTime = val.selectedValues;
                forms.endTimeStatus = false;
                const valTime = val.selectedValues.join('-');
                forms.endTimeStr = valTime;
                searchObj.type = '' as any;
              }}
            />
          )}
        </Popup>
      </div>
    );
  }
});
